from typing import Literal, Tuple, Union
from datetime import date

from sqlalchemy import select, delete, update, insert
from sqlalchemy import ScalarResult

from sqlalchemy.exc import NoResultFound

from ..database.db import async_session
from ..database.models import Apply


async def insert_apply(
    relation: int,
    applicant: str,
    recipient: str,
    group_chat_id: str | None,
    hello: str,
    setting: dict[str, str],
):
    session = async_session()
    res: ScalarResult[Apply] = await session.scalars(
        select(Apply).where(Apply.recipient == recipient, Apply.applicant == applicant)
    )

    try:
        apply = res.one()
        apply.hello = hello
        apply.setting = setting
        session.add(apply)
    except NoResultFound:
        apply = Apply(
            relation=relation,
            applicant=applicant,
            recipient=recipient,
            group_chat_id=group_chat_id,
            hello=hello,
            setting=setting,
        )
        session.add(apply)
    finally:
        await session.commit()
        await session.close()


async def select_apply_all(recipient: str) -> list[Apply]:
    session = async_session()
    res: ScalarResult[Apply] = await session.scalars(
        select(Apply)
        .where(Apply.recipient == recipient)
        .order_by(Apply.created_at.desc())
    )
    try:
        return list(res.all())
    except NoResultFound:
        return []
    finally:
        await session.close()


async def delete_apply(relation: int, applicant: str, recipient: str):
    session = async_session()
    await session.execute(
        delete(Apply).where(
            Apply.relation == relation,
            Apply.applicant == applicant,
            Apply.recipient == recipient,
        )
    )
    await session.commit()
    await session.close()
