from fastapi import Depends
from fastapi.responses import Response, JSONResponse
from jose import ExpiredSignatureError, JWTError

from .utils import token_handler as th


def verify_token(response: Response, token: str = Depends(th.oauth2_scheme)):
    try:
        token, _ = th.verify_signin_token(token)
        if token:
            response.headers['Authorization'] = token
    except ExpiredSignatureError:
        return JSONResponse({'code': 9999, 'msg': 'Token Expire'})
    except JWTError:
        return JSONResponse({'code': 9998, 'msg': 'Token Is Not Right'})
