from fastapi import FastAPI, Depends, WebSocket
from fastapi.staticfiles import StaticFiles

from .dependencies import verify_token
from .utils.email_code import smtp
from .utils.web_socket import manager
from .routers.signin import router as signin_router
from .routers.signup import router as signup_router
from .routers.user_profile import router as user_profile_router
from .routers.user_account import router as user_account_router
from .routers.search import router as search_router
from .routers.apply import router as apply_router
from .routers.contact import router as contact_router
from .routers.message import router as message_router


app = FastAPI()
app.include_router(signup_router)
app.include_router(signin_router)
app.include_router(user_profile_router, dependencies=[Depends(verify_token)])
app.include_router(user_account_router, dependencies=[Depends(verify_token)])
app.include_router(search_router, dependencies=[Depends(verify_token)])
app.include_router(apply_router, dependencies=[Depends(verify_token)])
app.include_router(contact_router, dependencies=[Depends(verify_token)])
app.include_router(
    message_router,
)

app.mount("/static", StaticFiles(directory="static"), name="static")


@app.on_event("shutdown")
def close_smtp():
    smtp.close()


@app.get("/")
async def main():
    return {"code": 10000, "msg": "hello world"}


@app.websocket("/ws/connect")
async def connect_websocket(websocket: WebSocket, id: str):
    await manager.connect(id, websocket)
