from datetime import datetime, timedelta
from typing import TypedDict, Tuple

from fastapi.security import OAuth2PasswordBearer

from jose import jwt, ExpiredSignatureError, JWTError

# openssl rand -hex 32
SECRET_KEY = '1c3c03b79d084f0c7b41ba11d1d9a4979f72d9fc6eaaaa0a855065e8a5be0468'
ALGORITHM = 'HS256'

oauth2_scheme = OAuth2PasswordBearer(tokenUrl='token')


class SigninClaim(TypedDict):
    sub: str
    iss: str
    iat: float
    exp: float
    device_id: str


def create_signin_token(id: str, device_id: str) -> str:
    claim: SigninClaim = {
        'sub': id,
        'iss': 'together',
        'iat': datetime.now().timestamp(),
        'exp': (datetime.now() + timedelta(days=23)).timestamp(),
        'device_id': device_id,
    }
    return jwt.encode(claim, SECRET_KEY, algorithm=ALGORITHM)


def verify_signin_token(token) -> Tuple[str | None,  str]:
    try:
        claim: SigninClaim = jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])
        if claim['exp'] - datetime.now().timestamp() <= 10*24*60*60:
            new_token = _prolong_token(token)
            return new_token, claim['sub']
        else:
            return None, claim['sub']
    except ExpiredSignatureError:
        print(f'{token} expire ========================')
        raise ExpiredSignatureError
    except JWTError:
        print(f'This token {token} is not sign by me ==========================')
        raise JWTError


def _prolong_token(claim: SigninClaim) -> str:
    claim['iat'] = datetime.now().timestamp()
    claim['exp'] = (datetime.now() + timedelta(days=30)).timestamp()
    token = jwt.encode(claim, SECRET_KEY, algorithm=ALGORITHM)
    return token
